/*--------------------------------------------------------------------------*/
/*                                                                          */
/*                                                                          */
/*      ------------         Bit-Bucket Software <no-Inc>                   */
/*      \ 10001101 /         Writers and Distributors of                    */
/*       \ 011110 /          No-Cost<no-tm> Software.                       */
/*        \ 1011 /                                                          */
/*         ------                                                           */
/*                                                                          */
/*  Copyright (C) 1987, 1988, 1989 by Robert Hartman and Vincent Perriello  */
/*                                                                          */
/*                                                                          */
/*               This module was written by Vince Perriello                 */
/*                                                                          */
/*                                                                          */
/*                    BinkleyTerm Modem Handler Module                      */
/*                                                                          */
/*                                                                          */
/*    For complete  details  of the licensing restrictions, please refer    */
/*    to the License  agreement,  which  is published in its entirety in    */
/*    the MAKEFILE and BT.C, and also contained in the file LICENSE.210.    */
/*                                                                          */
/*    USE  OF THIS FILE IS SUBJECT TO THE  RESTRICTIONS CONTAINED IN THE    */
/*    BINKLEYTERM  LICENSING  AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF    */
/*    THIS  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES,  OR IF YOU DO    */
/*    NOT HAVE THESE FILES,  YOU SHOULD  IMMEDIATELY CONTACT THE AUTHORS    */
/*    AT THE  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO    */
/*    USE   THIS  FILE  WITHOUT  HAVING   ACCEPTED  THE  TERMS  OF   THE    */
/*    BINKLEYTERM  LICENSING AGREEMENT,  OR SUCH OTHER  AGREEMENT AS YOU    */
/*    ARE ABLE TO REACH WITH THE AUTHORS.                                   */
/*                                                                          */
/*                                                                          */
/*    The Authors can be reached at the following addresses:                */
/*                                                                          */
/*    Robert C. Hartman                      Vincent E. Perriello           */
/*    Spark Software                         VEP Software                   */
/*    427-3 Amherst Street                   111 Carroll Street             */
/*    CS2032, Suite 232                      Naugatuck, CT 06770            */
/*    Nashua, NH 03061                                                      */
/*                                                                          */
/*    FidoNet 1:132/101                      FidoNet 1:141/491              */
/*    Data    (603) 888-8179                 Data    (203) 729-7569         */
/*                                                                          */
/*    Please feel free to contact us at any time to share your comments     */
/*    about our software and/or licensing policies.                         */
/*                                                                          */
/*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>

#include "com.h"
#include "xfer.h"
#include "zmodem.h"
#include "keybd.h"
#include "sbuf.h"
#include "sched.h"
#include "externs.h"
#include "prototyp.h"

static int dial_modem (char *);
static void phone_translate (char *, char *);
static char *get_response (long);
static int parse_response (char *);
static void empty_delay (void);

#define FAILURE 0
#define IGNORE	1
#define SUCCESS 2

struct resp_str
{
   char *resp;
   unsigned disp;
};

static struct resp_str mdm_resps[] = {
                                      {"RINGING", 1},
                                      {"RING", 3},
                                      {"CONNECT", 2},
                                      {"RRING", 1},
                                      {"BUSY", 0},
                                      {"VOICE", 0},
                                      {"ERROR", 0},
                                      {"OK", 0},
                                      {"NO CARRIER", 0},
/* "NO DIAL TONE" or "NO DIALTONE" */ {"NO DIAL", 0}, 
                                      {"DIAL TONE", 0},
                                      {NULL, 0}
};

static char *response_string = "                    ";

void try_2_connect (phnum)
char *phnum;
{
   long t1, timerset ();
   int j, k;

   for (j = 0; (j < poll_tries && !KEYPRESS ()); j++) /* do polltries or till keypress */
      {
      CLEAR_INBOUND ();
      k = dial_modem (phnum);

      if ((un_attended || doing_poll) && fullscreen)
         {
         ++hist.calls_made;
         sb_move (historywin, HIST_ATT_ROW, HIST_COL);
         sprintf (junk, "%-4d", hist.calls_made);
         sb_puts (historywin, junk);
         }

      if ((k > 0) || KEYPRESS ())
         break;
      t1 = timerset (200);
      while (!timeup (t1) && !KEYPRESS ())
         time_release ();                        /* pause for 2 seconds */
      }
   if (KEYPRESS ())                              /* If user's been busy */
      {
      if (!caller)
         FOSSIL_CHAR ();                         /* Eat the character   */
      if (!CARRIER)                              /* Abort if no carrier */
         {
         status_line ("!Connection attempt aborted");
         mdm_hangup ();
         }
      }
}

int try_1_connect (phnum)
char *phnum;
{
   int k;

   if ((k = dial_modem (phnum)) <= 0)
      {
      mdm_hangup ();
      }

   if ((un_attended || doing_poll) && fullscreen)
      {
      ++hist.calls_made;
      sb_move (historywin, HIST_ATT_ROW, HIST_COL);
      sprintf (junk, "%-4d", hist.calls_made);
      sb_puts (historywin, junk);
      }

   return (k);
}

static void phone_translate (number, translated)
char *number;
char *translated;
{
   PN_TRNS *p;

   strcpy (translated, number);
   for (p = pn_head; p != NULL; p = p->next)
      {
      if (strncmp (p->num, number, p->len) == 0)
         {
         sprintf (translated, "%s%s%s", p->pre, &(number[p->len]), p->suf);
         break;
         }
      }
}

static int dial_modem (number)
char *number;
{
   int resp;
   long t;
   char translated[50];
   extern long timerset ();

   phone_translate (number, translated);
   if (translated[0] == '\"')                    /* If it's a script          */
      return (do_script (translated));           /* then do it that way       */

   status_line (":Dialing %s", translated);
   if (un_attended && fullscreen)
      {
      do_ready ("Dialing");
      }

   /* First of all, if we have something, don't hang up on the guy! */
   if (!no_collide && CHAR_AVAIL ())
      return (-1);

   DTR_OFF ();                                   /* drop DTR to reset modem   */
   timer (20);                                   /* leave it down 2 seconds   */
   DTR_ON ();                                    /* then raise DTR again      */
   timer (5);                                    /* and wait .5 sec for modem */

   if (!no_collide && CHAR_AVAIL ())             /* If we have something
                                                  * here, return */
      return (-1);

   mdm_cmd_string (predial, 0);                  /* transmit the dial prefix  */
   mdm_cmd_string (translated, 0);               /* then the phone number     */
   mdm_cmd_string (postdial, 0);                 /* finally the dial suffix   */
   if (no_collide)
      CLEAR_INBOUND ();                          /* Throw out all echo to
                                                  * this point  */
   mdm_cmd_char (CR);                            /* terminate the string      */

   if (resp = modem_response (7500))             /* we got a good response,   */
      {
      if (resp == 3)                             /* Incoming ring to be
                                                  * processed higher up */
         return (-1);

      t = timerset (200);                        /* Wait up to 2 seconds      */
      while (!timeup (t))
         {                                       /* If carrier detect, AND    */
         if ((CHAR_AVAIL ()) && CARRIER)         /* some sign of life, */
            break;                               /* leave early...            */
         }
      return (CARRIER);                          /* Carrier should be on now  */
      }
   return (0);                                   /* no good */
}

static char *get_response (end_time)
long end_time;                                   /* timeout parameters        */
{
   char *p = response_string;                    /* points to character cell  */
   char c;                                       /* current modem character   */
   int count = 0;                                /* count of characters       */

   while ((count < 20)                           /* until we have 20 chars,   */
          && (!timeup (end_time))                /* or out of time,           */
          && (!KEYPRESS ()))                     /* or user gets impatient    */
      {
      if (!CHAR_AVAIL ())                        /* if nothing ready yet,     */
         {
         time_release ();
         continue;                               /* just process timeouts     */
         }
      c = (char) (MODEM_IN () & 0xff);           /* get a character           */
      if (c == '\r' || c == '\n')                /* if a line ending          */
         {
         if (count != 0)                         /* and we have something,    */
            break;                               /* get out                   */
         else continue;                          /* otherwise just keep going */
         }
      *p++ = c;                                  /* store the character       */
      ++count;                                   /* increment the counter     */
      }
   *p = '\0';                                    /* terminate the new string  */

   if (count != 0 && strnicmp (response_string, "AT", 2))
      {
      fancy_str (response_string);               /* make it pretty            */
      status_line ("\#%s", response_string);     /* pop it out on the screen  */
      }

   return (response_string);                     /* return the pointer        */
}

static int parse_response (response)
char *response;
{
   char *p;                                      /* temp character pointer    */
   register int i;                               /* array pointer             */

   for (i = 0; mdm_resps[i].resp != NULL; i++)   /* scan through array        */
      {
      p = mdm_resps[i].resp;                     /* point at possible
                                                  * response */
      if (strnicmp (response, p, strlen (p)) == 0)      /* if a match,               */
         return (mdm_resps[i].disp);             /* return disposition of it  */
      }
   return (1);                                   /* ignore all unknowns       */
}


int modem_response (ths)
int ths;                                         /* millisecs to wait         */
{
   long end_time;                                /* holds time at end of 2min */
   extern long timerset ();
   char *response;                               /* pointer to modem response */
   char *c, *skip_blanks ();                     /* miscellaneous pointer     */
   int result = IGNORE;                          /* result code               */

   /* If this modem doesn't differentiate between RING and RINGING */
   if (modemring)
      mdm_resps[0].disp = 1;

   end_time = timerset (ths);                    /* arm the timeout           */
   while ((result == IGNORE)                     /* until success or failure, */
          && (!timeup (end_time))                /* or out of time,           */
          && (!KEYPRESS ()))                     /* or user gets impatient    */
      {
      response = get_response (end_time);        /* get a response            */
      result = parse_response (response);        /* parse, determine status   */
      time_release ();
      }

   if (result == SUCCESS)                        /* Got to be a CONNECT msg   */
      {
      if (strnicmp (response, "connect", 7) == 0)/* if this is a CONNECT,     */
         {
         c = skip_blanks (&response[7]);         /* get past the blanks       */
         mdm_reliable[0] = '\0';                 /* Start with nothing        */
         if (*c == '\0')                         /* if nothing there,         */
            {
            set_baud (300, 1);                   /* say that it's 300 baud    */
            }
         else
            {
            set_baud (atoi (c), 1);              /* else do baudrate fallback */
            while (isdigit (*c))                 /* Get past digits           */
               ++c;
            c = skip_blanks (c);                 /* Get rid of blanks         */
            if (*c != '\0')                      /* We have "reliable" info.  */
               {
               strcpy (mdm_reliable, c);         /* Copy in the info          */
               }
            }

         }
      }

   return (result);                              /* timeout or failure or OK  */
}

void mdm_cmd_string (mdm_cmd, dospace)
char *mdm_cmd;
int dospace;
{
   register char *c;

   if (mdm_cmd == NULL)                          /* defense from shit         */
      return;

   for (c = mdm_cmd; *c; c++)
      {
      if (dospace || !isspace (*c))              /* don't output spaces       */
         mdm_cmd_char (*c);                      /* output the next character */
      }
}

static void empty_delay ()
{
   long t, timerset ();

   t = timerset (500);
   while ((!OUT_EMPTY ()) && (!timeup (t)))
      time_release ();                           /* wait for output to finish */
   if (!OUT_EMPTY ())
      {
/*      status_line ("!Timeout waiting for output to clear"); */
      MDM_DISABLE ();
      Cominit (port_ptr);
      MDM_ENABLE (btypes[baud].rate);
      DTR_ON ();
      CLEAR_OUTBOUND ();
      CLEAR_INBOUND ();
      if (un_attended && fullscreen)
         {
         sb_dirty ();
         sb_show ();
         }
      }
}

void mdm_cmd_char (outchr)
char outchr;
{

   switch (outchr)
      {
      case '\-':                                /* if it's a dash (phone no) */
         return;                                 /* ignore it                 */

      case '\|':                                /* if the CR character,      */
         outchr = CR;                            /* substitute a real CR here */
         break;

      case '.':                                 /* Substitute ',' for '.'    */
         outchr = ',';                           /* for compatibility         */
         break;

      case '\~':                                /* if the "delay" character, */
         empty_delay ();                         /* wait for buffer to clear, */
         timer (10);                             /* then wait 1 second        */
         return;                                 /* and return                */

      case '^':                                 /* Raise DTR                 */
         empty_delay ();                         /* wait for buffer to clear, */
         DTR_ON ();                              /* Turn on DTR               */
         return;                                 /* and return                */

      case 'v':                                 /* Lower DTR 		     */
         empty_delay ();                         /* wait for buffer to clear, */
         DTR_OFF ();                             /* Turn off DTR              */
         return;                                 /* and return                */

      case '`':                                 /* Short delay 		     */
         timer (1);                              /* short pause, .1 second    */
         return;                                 /* and return                */

      default:
         break;
      }

   SENDBYTE (outchr);                            /* then write the character  */

   if (outchr == CR)                             /* if it was a CR,           */
      {
      empty_delay ();
      timer (1);                                 /* allow .1 sec line quiet   */
      }
   else if (slowmodem)
      {
      timer (1);                                 /* wait .1 sec for output    */
      }
}

void mdm_hangup ()
{

   /*
    * First, if a dial command is in progress, try to get the modem to abort
    * it... 
    */

   CLEAR_OUTBOUND ();
   CLEAR_INBOUND ();

   if (un_attended && fullscreen)
      {
      do_ready ("HangUp ");
      }
   else
      {
      status_line ("#Modem hang up sequence");   /* Tell what we are doing */
      }

   mdm_init (modem_init);                        /* re-initialize the modem   */
   timer (5);                                    /* Wait another .5 sec       */

   set_xy ("");
   CLEAR_INBOUND ();                             /* then flush input and exit */
}

void mdm_init (str)
char *str;
{
   CLEAR_OUTBOUND ();
   CLEAR_INBOUND ();
   mdm_cmd_char (CR);                            /* output a CR, then         */
   DTR_OFF ();                                   /* Drop DTR to hangup        */
   timer (10);                                   /* Hold it down for 1 sec    */

   DTR_ON ();                                    /* Raise DTR,                */
   timer (5);                                    /* Then hold it up for .5
                                                  * sec */
   set_baud (max_baud, 0);

   mdm_cmd_char (' ');                           /* output a space            */
   mdm_cmd_char (CR);                            /* then another CR           */
   mdm_cmd_string (str, 0);                      /* then the modem init
                                                  * string */
   timer (5);                                    /* Hold DTR for .5 sec more  */
   CLEAR_INBOUND ();                             /* then flush input and exit */
}

void send_break (t)
int t;
{
   long t1;

   t1 = timerset (t);
   do_break (1);
   while (!timeup (t1))
      time_release ();
   do_break (0);
}
